@@ -1,5 +1,4 @@
 /* Options variables manipulation core */
-/* $Id: options.c,v 1.468.2.4 2005/04/05 21:08:40 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -17,13 +16,14 @@
 #include "config/options.h"
 #include "config/opttypes.h"
 #include "dialogs/status.h"
-#include "document/options.h"
+#include "document/document.h"
 #include "globhist/globhist.h"
 #include "intl/charsets.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/select.h"
-#include "main.h" /* shrink_memory() */
-#include "sched/session.h"
+#include "main/main.h" /* shrink_memory() */
+#include "main/select.h"
+#include "network/connection.h"
+#include "session/session.h"
 #include "terminal/color.h"
 #include "terminal/screen.h"
 #include "terminal/terminal.h"
@@ -74,7 +74,7 @@ static void free_options_tree(struct list_head *, int recursive);
 /* Detect ending '.' (and some others) in options captions.
  * It will emit a message in debug mode only. --Zas */
 
-#define bad_punct(c) (c != ')' && !isquote(c) && ispunct(c))
+#define bad_punct(c) (c != ')' && c != '>' && !isquote(c) && ispunct(c))
 
 static void
 check_caption(unsigned char *caption)
@@ -91,7 +91,7 @@ check_caption(unsigned char *caption)
 	if (isspace(c) || bad_punct(c))
 		DBG("bad char at end of caption [%s]", caption);
 
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 	caption = gettext(caption);
 	len = strlen(caption);
 	if (!len) return;
@@ -119,7 +119,7 @@ check_description(unsigned char *desc)
 	if (isspace(c))
 		DBG("bad char at end of description [%s]", desc);
 
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 	desc = gettext(desc);
 	len = strlen(desc);
 	if (!len) return;
@@ -157,7 +157,14 @@ debug_check_option_syntax(struct option *option)
 /* Ugly kludge */
 static int no_autocreate = 0;
 
-/* Get record of option of given name, or NULL if there's no such option. */
+/** Get record of option of given name, or NULL if there's no such option.
+ *
+ * If the specified option is an ::OPT_ALIAS, this function returns the
+ * alias, rather than the option to which the alias refers.  It must
+ * work this way because the alias may have the ::OPT_ALIAS_NEGATE flag.
+ * Instead, if the caller tries to read or set the value of the alias,
+ * the functions associated with ::OPT_ALIAS will forward the operation
+ * to the underlying option.  However, see indirect_option().  */
 struct option *
 get_opt_rec(struct option *tree, unsigned char *name_)
 {
@@ -241,6 +248,27 @@ get_opt_rec_real(struct option *tree, unsigned char *name)
 	return opt;
 }
 
+/** If @a opt is an alias, return the option to which it refers.
+ *
+ * @warning Because the alias may have the ::OPT_ALIAS_NEGATE flag,
+ * the caller must not access the value of the returned option as if
+ * it were also the value of the alias.  However, it is safe to access
+ * flags such as ::OPT_MUST_SAVE and ::OPT_DELETED.  */
+struct option *
+indirect_option(struct option *alias)
+{
+	struct option *real;
+
+	if (alias->type != OPT_ALIAS) return alias; /* not an error */
+
+	real = get_opt_rec(config_options, alias->value.string);
+	assertm(real != NULL, "%s aliased to unknown option %s!",
+		alias->name, alias->value.string);
+	if_assert_failed return alias;
+
+	return real;
+}
+
 /* Fetch pointer to value of certain option. It is guaranteed to never return
  * NULL. Note that you are supposed to use wrapper get_opt(). */
 union option_value *
@@ -282,7 +310,7 @@ get_opt_(
 	case OPT_LONG:
 		if (opt->value.number < opt->min
 		    || opt->value.number > opt->max)
-			elinks_internal("Option %s has invalid value!", name);
+			elinks_internal("Option %s has invalid value %d!", name, opt->value.number);
 		break;
 	case OPT_COMMAND:
 		if (!opt->value.command)
@@ -430,16 +458,16 @@ add_opt_rec(struct option *tree, unsigned char *path, struct option *option)
 static inline struct listbox_item *
 init_option_listbox_item(struct option *option)
 {
-	struct listbox_item *box = mem_calloc(1, sizeof(*box));
+	struct listbox_item *item = mem_calloc(1, sizeof(*item));
 
-	if (!box) return NULL;
+	if (!item) return NULL;
 
-	init_list(box->child);
-	box->visible = 1;
-	box->udata = option;
-	box->type = (option->type == OPT_TREE) ? BI_FOLDER : BI_LEAF;
+	init_list(item->child);
+	item->visible = 1;
+	item->udata = option;
+	item->type = (option->type == OPT_TREE) ? BI_FOLDER : BI_LEAF;
 
-	return box;
+	return item;
 }
 
 struct option *
@@ -515,40 +543,15 @@ add_opt(struct option *tree, unsigned char *path, unsigned char *capt,
 	return option;
 }
 
-/* The namespace may start to seem a bit chaotic here; it indeed is, maybe the
- * function names above should be renamed and only macros should keep their old
- * short names. */
-/* The simple rule I took as an apologize is that functions which take already
- * completely filled (struct option *) have long name and functions which take
- * only option specs have short name. */
-
 static void
-delete_option_do(struct option *option, int recursive)
+done_option(struct option *option)
 {
-	if (option->next) del_from_list(option);
-
-	if (recursive == -1) {
-		ERROR("Orphaned option %s", option->name);
-	}
-
 	switch (option->type) {
 		case OPT_STRING:
 			mem_free_if(option->value.string);
 			break;
 		case OPT_TREE:
-			if (!option->value.tree) break;
-			if (!recursive && !list_empty(*option->value.tree)) {
-				if (option->flags & OPT_AUTOCREATE) {
-					recursive = 1;
-				} else {
-					ERROR("Orphaned unregistered "
-						"option in subtree %s!",
-						option->name);
-					recursive = -1;
-				}
-			}
-			free_options_tree(option->value.tree, recursive);
-			mem_free(option->value.tree);
+			mem_free_if(option->value.tree);
 			break;
 		default:
 			break;
@@ -568,6 +571,43 @@ delete_option_do(struct option *option, int recursive)
 	}
 }
 
+/* The namespace may start to seem a bit chaotic here; it indeed is, maybe the
+ * function names above should be renamed and only macros should keep their old
+ * short names. */
+/* The simple rule I took as an apologize is that functions which take already
+ * completely filled (struct option *) have long name and functions which take
+ * only option specs have short name. */
+
+static void
+delete_option_do(struct option *option, int recursive)
+{
+	if (option->next) {
+		del_from_list(option);
+		option->prev = option->next = NULL;
+	}
+
+	if (recursive == -1) {
+		ERROR("Orphaned option %s", option->name);
+	}
+
+	if (option->type == OPT_TREE && option->value.tree
+	    && !list_empty(*option->value.tree)) {
+		if (!recursive) {
+			if (option->flags & OPT_AUTOCREATE) {
+				recursive = 1;
+			} else {
+				ERROR("Orphaned unregistered "
+					"option in subtree %s!",
+					option->name);
+				recursive = -1;
+			}
+		}
+		free_options_tree(option->value.tree, recursive);
+	}
+
+	done_option(option);
+}
+
 void
 mark_option_as_deleted(struct option *option)
 {
@@ -580,10 +620,7 @@ mark_option_as_deleted(struct option *option)
 			mark_option_as_deleted(unmarked);
 	}
 
-	if (option->box_item) {
-		done_listbox_item(&option_browser, option->box_item);
-		option->box_item = NULL;
-	}
+	option->box_item->visible = 0;
 
 	option->flags |= (OPT_TOUCHED | OPT_DELETED);
 }
@@ -651,6 +688,11 @@ register_autocreated_options(void)
 	get_opt_int("terminal.xterm-color.type") = 1;
 	get_opt_int("terminal.xterm-color.colors") = COLOR_MODE_16;
 	get_opt_bool("terminal.xterm-color.underline") = 1;
+#ifdef CONFIG_88_COLORS
+	get_opt_int("terminal.xterm-88color.type") = 1;
+	get_opt_int("terminal.xterm-88color.colors") = COLOR_MODE_88;
+	get_opt_bool("terminal.xterm-88color.underline") = 1;
+#endif
 #ifdef CONFIG_256_COLORS
 	get_opt_int("terminal.xterm-256color.type") = 1;
 	get_opt_int("terminal.xterm-256color.colors") = COLOR_MODE_256;
@@ -710,27 +752,35 @@ register_change_hooks(struct change_hook_info *change_hooks)
 }
 
 void
-unmark_options_tree(struct list_head *tree)
+prepare_mustsave_flags(struct list_head *tree, int set_all)
 {
 	struct option *option;
 
 	foreach (option, *tree) {
-		option->flags &= ~OPT_WATERMARK;
+		/* XXX: OPT_LANGUAGE shouldn't have any bussiness
+		 * here, but we're just weird in that area. */
+		if (set_all
+		    || (option->flags & (OPT_TOUCHED | OPT_DELETED))
+		    || option->type == OPT_LANGUAGE)
+			option->flags |= OPT_MUST_SAVE;
+		else
+			option->flags &= ~OPT_MUST_SAVE;
+
 		if (option->type == OPT_TREE)
-			unmark_options_tree(option->value.tree);
+			prepare_mustsave_flags(option->value.tree, set_all);
 	}
 }
 
 void
-watermark_deleted_options(struct list_head *tree)
+untouch_options(struct list_head *tree)
 {
 	struct option *option;
 
 	foreach (option, *tree) {
-		if (option->flags & OPT_DELETED)
-			option->flags |= OPT_WATERMARK;
-		else if (option->type == OPT_TREE)
-			watermark_deleted_options(option->value.tree);
+		option->flags &= ~OPT_TOUCHED;
+
+		if (option->type == OPT_TREE)
+			untouch_options(option->value.tree);
 	}
 }
 
@@ -743,7 +793,7 @@ check_nonempty_tree(struct list_head *options)
 		if (opt->type == OPT_TREE) {
 			if (check_nonempty_tree(opt->value.tree))
 				return 1;
-		} else if (!(opt->flags & OPT_WATERMARK)) {
+		} else if (opt->flags & OPT_MUST_SAVE) {
 			return 1;
 		}
 	}
@@ -763,14 +813,14 @@ smart_config_string(struct string *str, int print_comment, int i18n,
 		int do_print_comment = 1;
 
 		if (option->flags & OPT_HIDDEN ||
-		    option->flags & OPT_WATERMARK ||
 		    option->type == OPT_ALIAS ||
 		    !strcmp(option->name, "_template_"))
 			continue;
 
 		/* Is there anything to be printed anyway? */
 		if (option->type == OPT_TREE
-		    && !check_nonempty_tree(option->value.tree))
+		    ? !check_nonempty_tree(option->value.tree)
+		    : !(option->flags & OPT_MUST_SAVE))
 			continue;
 
 		/* We won't pop out the description when we're in autocreate
@@ -836,10 +886,6 @@ smart_config_string(struct string *str, int print_comment, int i18n,
 
 			fn(str, option, path, depth, /*pc*/1, 3, i18n);
 		}
-
-		/* TODO: We should maybe clear the touched flag only when really
-		 * saving the stuff...? --pasky */
-		option->flags &= ~OPT_TOUCHED;
 	}
 }
 
@@ -854,17 +900,15 @@ change_hook_cache(struct session *ses, struct option *current, struct option *ch
 static int
 change_hook_connection(struct session *ses, struct option *current, struct option *changed)
 {
-	register_bottom_half((void (*)(void *)) check_queue, NULL);
+	register_check_queue();
 	return 0;
 }
 
 static int
 change_hook_html(struct session *ses, struct option *current, struct option *changed)
 {
-	draw_formatted(ses, 1);
-	load_frames(ses, ses->doc_view);
-	process_file_requests(ses);
-	print_screen_status(ses);
+	foreach (ses, sessions) ses->tab->resize = 1;
+
 	return 0;
 }
 
@@ -878,15 +922,7 @@ change_hook_insert_mode(struct session *ses, struct option *current, struct opti
 static int
 change_hook_active_link(struct session *ses, struct option *current, struct option *changed)
 {
-	if (!global_doc_opts) return 0;
-
-	global_doc_opts->active_link_fg = get_opt_color("document.browse.links.active_link.colors.text");
-	global_doc_opts->active_link_bg = get_opt_color("document.browse.links.active_link.colors.background");
-	global_doc_opts->color_active_link = get_opt_bool("document.browse.links.active_link.enable_color");
-	global_doc_opts->invert_active_link = get_opt_bool("document.browse.links.active_link.invert");
-	global_doc_opts->underline_active_link = get_opt_bool("document.browse.links.active_link.underline");
-	global_doc_opts->bold_active_link = get_opt_bool("document.browse.links.active_link.bold");
-
+	update_cached_document_options();
 	return 0;
 }
 
@@ -912,6 +948,8 @@ update_visibility(struct list_head *tree, int show)
 	struct option *opt;
 
 	foreach (opt, *tree) {
+		if (opt->flags & OPT_DELETED) continue;
+
 		if (!strcmp(opt->name, "_template_")) {
 			if (opt->box_item)
 				opt->box_item->visible = (show & 1);
@@ -935,6 +973,19 @@ update_options_visibility(void)
 			  get_opt_bool("config.show_template"));
 }
 
+void
+toggle_option(struct session *ses, struct option *option)
+{
+	long number = option->value.number + 1;
+
+	assert(option->type == OPT_BOOL || option->type == OPT_INT);
+	assert(option->max);
+
+	/* TODO: call change hooks. --jonas */
+	option->value.number = (number <= option->max) ? number : option->min;
+	option_changed(ses, option, option);
+}
+
 static int
 change_hook_stemplate(struct session *ses, struct option *current, struct option *changed)
 {
@@ -945,7 +996,7 @@ change_hook_stemplate(struct session *ses, struct option *current, struct option
 static int
 change_hook_language(struct session *ses, struct option *current, struct option *changed)
 {
-#ifdef ENABLE_NLS
+#ifdef CONFIG_NLS
 	set_language(changed->value.number);
 #endif
 	return 0;
@@ -970,6 +1021,30 @@ static struct change_hook_info change_hooks[] = {
 	{ NULL,				NULL },
 };
 
+void
+call_change_hooks(struct session *ses, struct option *current, struct option *option)
+{
+	/* This boolean thing can look a little weird - it
+	 * basically says that we should proceed when there's
+	 * no change_hook or there's one and its return value
+	 * was zero. */
+	while (current && (!current->change_hook ||
+		!current->change_hook(ses, current, option))) {
+		if (!current->root)
+			break;
+
+		current = current->root;
+	}
+}
+
+void
+option_changed(struct session *ses, struct option *current, struct option *option)
+{
+	option->flags |= OPT_TOUCHED;
+	/* Notify everyone out there! */
+	call_change_hooks(ses, current, option);
+}
+
 int
 commit_option_values(struct option_resolver *resolvers,
 		     struct option *root, union option_value *values, int size)
@@ -987,10 +1062,22 @@ commit_option_values(struct option_resolver *resolvers,
 		if (memcmp(&option->value, &values[id], sizeof(union option_value))) {
 			option->value = values[id];
 			option->flags |= OPT_TOUCHED;
+			/* Speed hack: Directly call the change-hook for each
+			 * option in resolvers and later call call_change_hooks
+			 * on the root; if we were to call call_change_hooks
+			 * on each option in resolvers, we would end up calling
+			 * the change-hooks of root, its parents, its
+			 * grandparents, and so on for each option in resolvers
+			 * because call_change_hooks is recursive. -- Miciah */
+			if (option->change_hook)
+				option->change_hook(NULL, option, NULL);
 			touched++;
 		}
 	}
 
+	/* See above 'Speed hack' comment. */
+	call_change_hooks(NULL, root, NULL);
+
 	return touched;
 }
 
