@@ -1,5 +1,4 @@
 /* HTML renderer */
-/* $Id: renderer.c,v 1.140.2.4 2005/09/14 12:16:54 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -22,17 +21,19 @@
 #include "document/renderer.h"
 #include "document/view.h"
 #include "ecmascript/ecmascript.h"
+#include "encoding/encoding.h"
 #include "intl/charsets.h"
-#include "main.h"
+#include "main/main.h"
+#include "main/object.h"
 #include "protocol/header.h"
 #include "protocol/protocol.h"
 #include "protocol/uri.h"
-#include "sched/session.h"
+#include "session/location.h"
+#include "session/session.h"
 #include "terminal/terminal.h"
 #include "terminal/window.h"
 #include "util/error.h"
 #include "util/memory.h"
-#include "util/object.h"
 #include "util/string.h"
 #include "viewer/text/view.h"
 #include "viewer/text/vs.h"
@@ -58,7 +59,7 @@ add_snippets(struct ecmascript_interpreter *interpreter,
 
 #ifdef CONFIG_LEDS
 	if (list_empty(*queued_snippets) && interpreter->vs->doc_view->session)
-		interpreter->vs->doc_view->session->status.ecmascript_led->value = '-';
+		unset_led_value(interpreter->vs->doc_view->session->status.ecmascript_led);
 #endif
 
 	if (list_empty(*doc_snippets) || !get_opt_bool("ecmascript.enable"))
@@ -204,7 +205,6 @@ render_encoded_document(struct cache_entry *cached, struct document *document)
 {
 	struct uri *uri = cached->uri;
 	enum stream_encoding encoding = ENCODING_NONE;
-	unsigned char *extension;
 	struct fragment *fragment = get_cache_fragment(cached);
 	struct string buffer = INIT_STRING("", 0);
 
@@ -215,30 +215,38 @@ render_encoded_document(struct cache_entry *cached, struct document *document)
 		buffer.length = fragment->length;
 	}
 
-	extension = get_extension_from_uri(uri);
-	if (extension) {
-		encoding = guess_encoding(extension);
-		mem_free(extension);
-	}
+	if (uri->protocol != PROTOCOL_FILE) {
+		unsigned char *extension = get_extension_from_uri(uri);
 
-	if (encoding != ENCODING_NONE) {
-		int length = 0;
-		unsigned char *source;
+		if (extension) {
+			encoding = guess_encoding(extension);
+			mem_free(extension);
+		}
 
-		source = decode_encoded_buffer(encoding, buffer.source,
+		if (encoding != ENCODING_NONE) {
+			int length = 0;
+			unsigned char *source;
+
+			source = decode_encoded_buffer(encoding, buffer.source,
 					       buffer.length, &length);
-		if (source) {
-			buffer.source = source;
-			buffer.length = length;
-		} else {
-			encoding = ENCODING_NONE;
+			if (source) {
+				buffer.source = source;
+				buffer.length = length;
+			} else {
+				encoding = ENCODING_NONE;
+			}
 		}
 	}
 
 	if (document->options.plain) {
 #ifdef CONFIG_DOM
 		if (cached->content_type
-		    && !strlcasecmp("text/html", 9, cached->content_type, -1))
+		    && (!strcasecmp("text/html", cached->content_type)
+			|| !strcasecmp("application/xhtml+xml", cached->content_type)
+		        || !strcasecmp("application/rss+xml", cached->content_type)
+		        || !strcasecmp("application/xbel+xml", cached->content_type)
+		        || !strcasecmp("application/x-xbel", cached->content_type)
+		        || !strcasecmp("application/xbel", cached->content_type)))
 			render_dom_document(cached, document, &buffer);
 		else
 #endif
@@ -265,7 +273,7 @@ render_document(struct view_state *vs, struct document_view *doc_view,
 	if_assert_failed return;
 
 #if 0
-	DBG("(Re%d)Rendering %s on doc_view %p [%s] while attaching it to %p",
+	DBG("(Re%u)Rendering %s on doc_view %p [%s] while attaching it to %p",
 	    options->gradual_rerendering, struri(vs->uri),
 	    doc_view, doc_view->name, vs);
 #endif
