@@ -57,7 +57,7 @@ const JSClass history_class = {
 	JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub
 };
 
-const JSFunctionSpec history_funcs[] = {
+const spidermonkeyFunctionSpec history_funcs[] = {
 	{ "back",		history_back,		0 },
 	{ "forward",		history_forward,	0 },
 	{ "go",			history_go,		1 },
@@ -158,7 +158,8 @@ location_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 
 	if (!JSVAL_IS_INT(id))
 		return JS_TRUE;
@@ -170,8 +171,8 @@ location_get_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 		astring_to_jsval(ctx, vp, get_uri_string(vs->uri, URI_ORIGINAL));
 		break;
 	default:
-		/* Unrecognized property ID; someone is using the
-		 * object as an array.  SMJS builtin classes (e.g.
+		/* Unrecognized integer property ID; someone is using
+		 * the object as an array.  SMJS builtin classes (e.g.
 		 * js_RegExpClass) just return JS_TRUE in this case
 		 * and leave *@vp unchanged.  Do the same here.
 		 * (Actually not quite the same, as we already used
@@ -199,7 +200,8 @@ location_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 	assert(JS_InstanceOf(ctx, parent_win, (JSClass *) &window_class, NULL));
 	if_assert_failed return JS_FALSE;
 
-	vs = JS_GetPrivate(ctx, parent_win); /* from @window_class */
+	vs = JS_GetInstancePrivate(ctx, parent_win,
+				   (JSClass *) &window_class, NULL);
 	doc_view = vs->doc_view;
 
 	if (!JSVAL_IS_INT(id))
@@ -216,7 +218,7 @@ location_set_property(JSContext *ctx, JSObject *obj, jsval id, jsval *vp)
 
 static JSBool location_toString(JSContext *ctx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 
-const JSFunctionSpec location_funcs[] = {
+const spidermonkeyFunctionSpec location_funcs[] = {
 	{ "toString",		location_toString,	0 },
 	{ "toLocaleString",	location_toString,	0 },
 	{ NULL }
