@@ -1,10 +1,9 @@
-/* $Id: default.h,v 1.3 2004/06/22 21:49:49 jonas Exp $ */
 
 #ifndef EL__MIME_BACKEND_DEFAULT_H
 #define EL__MIME_BACKEND_DEFAULT_H
 
+#include "main/module.h"
 #include "mime/backend/common.h"
-#include "modules/module.h"
 
 extern struct mime_backend default_mime_backend;
 extern struct module default_mime_module;
