@@ -1,10 +1,9 @@
-/* $Id: mimetypes.h,v 1.4 2004/07/15 00:43:06 jonas Exp $ */
 
 #ifndef EL__MIME_BACKEND_MIMETYPES_H
 #define EL__MIME_BACKEND_MIMETYPES_H
 
+#include "main/module.h"
 #include "mime/backend/common.h"
-#include "modules/module.h"
 
 extern struct mime_backend mimetypes_mime_backend;
 extern struct module mimetypes_mime_module;
