@@ -1,5 +1,4 @@
 /* UNIX system-specific routines. */
-/* $Id: unix.c,v 1.21.4.2 2005/05/01 21:15:32 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -13,7 +12,7 @@
 
 #include "elinks.h"
 
-#include "lowlevel/select.h"
+#include "main/select.h"
 #include "osdep/unix/unix.h"
 #include "osdep/osdep.h"
 #include "terminal/event.h"
@@ -35,38 +34,39 @@ gpm_mouse_in(struct gpm_mouse_spec *gms)
 {
 	Gpm_Event gev;
 	struct term_event ev;
+	struct term_event_mouse mouse;
 
 	if (Gpm_GetEvent(&gev) <= 0) {
 		clear_handlers(gms->h);
 		return;
 	}
 
-	ev.ev = EVENT_MOUSE;
-	ev.info.mouse.x = int_max(gev.x - 1, 0);
-	ev.info.mouse.y = int_max(gev.y - 1, 0);
+	mouse.x = int_max(gev.x - 1, 0);
+	mouse.y = int_max(gev.y - 1, 0);
 
 	if (gev.buttons & GPM_B_LEFT)
-		ev.info.mouse.button = B_LEFT;
+		mouse.button = B_LEFT;
 	else if (gev.buttons & GPM_B_MIDDLE)
-		ev.info.mouse.button = B_MIDDLE;
+		mouse.button = B_MIDDLE;
 	else if (gev.buttons & GPM_B_RIGHT)
-		ev.info.mouse.button = B_RIGHT;
+		mouse.button = B_RIGHT;
 	else
 		return;
 
 	if (gev.type & GPM_DOWN)
-		ev.info.mouse.button |= B_DOWN;
+		mouse.button |= B_DOWN;
 	else if (gev.type & GPM_UP)
-		ev.info.mouse.button |= B_UP;
+		mouse.button |= B_UP;
 	else if (gev.type & GPM_DRAG)
-		ev.info.mouse.button |= B_DRAG;
+		mouse.button |= B_DRAG;
 	else
 		return;
 
+	set_mouse_term_event(&ev, mouse.x, mouse.y, mouse.button);
 	gms->fn(gms->data, (char *) &ev, sizeof(ev));
 }
 
-int
+static int
 init_mouse(int cons, int suspend)
 {
 	Gpm_Connect conn;
@@ -79,7 +79,7 @@ init_mouse(int cons, int suspend)
 	return Gpm_Open(&conn, cons);
 }
 
-int
+static int
 done_mouse(void)
 {
 	return Gpm_Close();
@@ -101,7 +101,7 @@ handle_mouse(int cons, void (*fn)(void *, unsigned char *, int),
 	gms->cons = cons;
 	gms->fn = fn;
 	gms->data = data;
-	set_handlers(h, (void (*)(void *)) gpm_mouse_in, NULL, NULL, gms);
+	set_handlers(h, (select_handler_T) gpm_mouse_in, NULL, NULL, gms);
 
 	return gms;
 }
@@ -141,7 +141,7 @@ resume_mouse(void *h)
 	gms->h = init_mouse(gms->cons, 0);
 	if (gms->h < 0) return;
 
-	set_handlers(gms->h, (void (*)(void *)) gpm_mouse_in, NULL, NULL, gms);
+	set_handlers(gms->h, (select_handler_T) gpm_mouse_in, NULL, NULL, gms);
 }
 
 #endif
