@@ -1,5 +1,4 @@
 /* Gopher access protocol (RFC 1436) */
-/* $Id: gopher.c,v 1.31.2.2 2005/05/01 21:04:46 jonas Exp $ */
 
 /* Based on version of HTGopher.c in the lynx tree.
  *
@@ -33,12 +32,12 @@
 
 #include "cache/cache.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/connect.h"
-#include "modules/module.h"
+#include "main/module.h"
+#include "network/connection.h"
+#include "network/socket.h"
 #include "protocol/gopher/gopher.h"
 #include "protocol/protocol.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
 #include "util/conv.h"
 #include "util/memory.h"
 #include "util/string.h"
@@ -141,7 +140,7 @@ get_gopher_entity_info(enum gopher_entity type)
 {
 	int entry;
 
-	for (entry = 0; entry < sizeof(gopher_entity_info) - 1; entry++)
+	for (entry = 0; entry < sizeof_array(gopher_entity_info) - 1; entry++)
 		if (gopher_entity_info[entry].type == type)
 			return &gopher_entity_info[entry];
 
@@ -283,7 +282,7 @@ init_gopher_connection_info(struct connection *conn)
 
 	/* Get entity type, and selector string. */
 	/* Pick up gopher_entity */
-	if (selectorlen > 0) {
+	if (selectorlen > 1 && selector[1] == '/') {
 		entity = *selector++;
 		selectorlen--;
 	}
@@ -297,7 +296,7 @@ init_gopher_connection_info(struct connection *conn)
 	 * _after_ the Gopher entity. If the <entity-char> '/' combo is not
 	 * found assume that the whole URI data part is the selector. */
 	entity_info = get_gopher_entity_info(entity);
-	if (entity_info->type == GOPHER_UNKNOWN) {
+	if (entity_info->type == GOPHER_UNKNOWN && entity != GOPHER_UNKNOWN) {
 		selector--;
 		selectorlen++;
 	}
@@ -346,12 +345,10 @@ end_gopher_connection(struct connection *conn, enum connection_state state)
 			conn->from -= 3;
 		}
 #endif
-		truncate_entry(conn->cached, conn->from, 1);
-
-		conn->cached->incomplete = 0;
+		normalize_cache_entry(conn->cached, conn->from);
 	}
 
-	abort_conn_with_state(conn, state);
+	abort_connection(conn, state);
 }
 
 
@@ -408,13 +405,13 @@ encode_selector_string(struct string *buffer, unsigned char *selector)
 	 * two successive ones. */
 	while ((slashes = strstr(selector, "//"))) {
 		*slashes = 0;
-		encode_uri_string(buffer, selector, 0);
-		encode_uri_string(buffer, "//", 1);
+		encode_uri_string(buffer, selector, -1, 0);
+		encode_uri_string(buffer, "//", 2, 1);
 		*slashes = '/';
 		selector = slashes + 2;
 	}
 
-	encode_uri_string(buffer, selector, 0);
+	encode_uri_string(buffer, selector, -1, 0);
 }
 
 static void
@@ -521,7 +518,8 @@ add_gopher_menu_line(struct string *buffer, unsigned char *line)
 	default:
 	{
 		struct string address;
-		unsigned char *format = *selector ? "%s://%s@%s/" : "%s://%s%s/";
+		unsigned char *format = selector && *selector
+				      ? "%s://%s@%s/" : "%s://%s%s/";
 
 		/* If port is defined it means that both @selector and @host
 		 * was correctly parsed. */
@@ -622,7 +620,7 @@ read_gopher_directory_data(struct connection *conn, struct read_buffer *rb)
 		mem_free_if(where);
 	}
 
-	while ((end = get_gopher_line_end(rb->data, rb->len))) {
+	while ((end = get_gopher_line_end(rb->data, rb->length))) {
 		unsigned char *line = check_gopher_last_line(rb->data, end);
 
 		/* Break on line with a dot by itself */
@@ -636,7 +634,7 @@ read_gopher_directory_data(struct connection *conn, struct read_buffer *rb)
 		kill_buffer_data(rb, end - rb->data);
 	}
 
-	if (state != S_TRANS || rb->close == READ_BUFFER_END)
+	if (state != S_TRANS || conn->socket->state == SOCKET_CLOSED)
 		add_to_string(&buffer,
 			"</pre>\n"
 			"</body>\n"
@@ -715,15 +713,14 @@ init_gopher_index_cache_entry(struct connection *conn)
 
 
 static void
-read_gopher_response_data(struct connection *conn, struct read_buffer *rb)
+read_gopher_response_data(struct socket *socket, struct read_buffer *rb)
 {
+	struct connection *conn = socket->conn;
 	struct gopher_connection_info *gopher = conn->info;
 	enum connection_state state = S_TRANS;
 
 	assert(gopher && gopher->entity);
 
-	set_connection_timeout(conn);
-
 	if (!conn->cached && !init_gopher_cache_entry(conn)) {
 		end_gopher_connection(conn, S_OUT_OF_MEM);
 		return;
@@ -761,16 +758,16 @@ read_gopher_response_data(struct connection *conn, struct read_buffer *rb)
 	default:
 		/* Add the received data as a new cache entry fragment and do
 		 * the connection data accounting. */
-		add_fragment(conn->cached, conn->from, rb->data, rb->len);
+		add_fragment(conn->cached, conn->from, rb->data, rb->length);
 
-		conn->received += rb->len;
-		conn->from     += rb->len;
+		conn->received += rb->length;
+		conn->from     += rb->length;
 
-		kill_buffer_data(rb, rb->len);
+		kill_buffer_data(rb, rb->length);
 	}
 
 	/* Has the transport layer forced a shut down? */
-	if (rb->close == READ_BUFFER_END) {
+	if (socket->state == SOCKET_CLOSED) {
 		state = S_OK;
 	}
 
@@ -779,34 +776,18 @@ read_gopher_response_data(struct connection *conn, struct read_buffer *rb)
 		return;
 	}
 
-	read_from_socket(conn, &conn->socket, rb, read_gopher_response_data);
-	set_connection_state(conn, S_TRANS);
+	read_from_socket(conn->socket, rb, S_TRANS, read_gopher_response_data);
 }
 
 
 static void
-receive_gopher_response(struct connection *conn)
-{
-	struct read_buffer *rb = alloc_read_buffer(conn);
-
-	if (!rb) return;
-
-	set_connection_timeout(conn);
-
-	rb->close = READ_BUFFER_END_ONCLOSE;
-	read_from_socket(conn, &conn->socket, rb, read_gopher_response_data);
-}
-
-static void
-send_gopher_command(struct connection *conn)
+send_gopher_command(struct socket *socket)
 {
+	struct connection *conn = socket->conn;
 	struct gopher_connection_info *gopher = conn->info;
 
-	write_to_socket(conn, &conn->socket,
-			gopher->command, gopher->commandlen,
-			receive_gopher_response);
-
-	set_connection_state(conn, S_SENT);
+	request_from_socket(socket, gopher->command, gopher->commandlen,
+			    S_SENT, SOCKET_END_ONCLOSE, read_gopher_response_data);
 }
 
 
@@ -840,9 +821,7 @@ gopher_protocol_handler(struct connection *conn)
 			/* FIXME: redirect_cache() */
 			end_gopher_connection(conn, S_OK);
 		}
-		break;
 #endif
-	default:
 		break;
 	}
 
@@ -854,7 +833,7 @@ gopher_protocol_handler(struct connection *conn)
 	}
 
 	/* Set up a socket to the server for the data */
-	set_connection_timeout(conn);
 	conn->from = 0;
-	make_connection(conn, &conn->socket, send_gopher_command);
+	make_connection(conn->socket, conn->uri, send_gopher_command,
+			conn->cache_mode >= CACHE_MODE_FORCE_RELOAD);
 }
