@@ -1,5 +1,4 @@
 /* HTTP response codes */
-/* $Id: codes.c,v 1.34.6.3 2005/04/05 21:08:42 jonas Exp $ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE /* Needed for asprintf() */
@@ -13,11 +12,11 @@
 
 #include "cache/cache.h"
 #include "intl/gettext/libintl.h"
+#include "network/connection.h"
 #include "protocol/http/codes.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
-#include "sched/session.h"
-#include "sched/task.h"
+#include "session/session.h"
+#include "session/task.h"
 #include "terminal/terminal.h"
 #include "terminal/window.h"
 #include "util/snprintf.h"
@@ -74,8 +73,6 @@ static struct http_code http_code[] = {
 	{ 505, "HTTP Version Not Supported" },
 };
 
-#define count(T) (sizeof(T)/sizeof(*(T)))
-
 static int
 compare_http_codes(const void *key, const void *element)
 {
@@ -89,7 +86,7 @@ static unsigned char *
 http_code_to_string(int code)
 {
 	struct http_code *element = bsearch((void *) (long) code, http_code,
-					    count(http_code),
+					    sizeof_array(http_code),
 					    sizeof(*element),
 					    compare_http_codes);
 
