@@ -1,5 +1,4 @@
 /* Connection and data transport handling */
-/* $Id: connection.c,v 1.3.4.4 2005/05/01 22:09:22 jonas Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -13,15 +12,15 @@
 
 #include "cache/cache.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/connect.h"
-#include "modules/module.h"
+#include "main/module.h"
+#include "network/connection.h"
+#include "network/socket.h"
 #include "protocol/nntp/codes.h"
 #include "protocol/nntp/connection.h"
 #include "protocol/nntp/nntp.h"
 #include "protocol/nntp/response.h"
 #include "protocol/protocol.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
 #include "util/memory.h"
 #include "util/string.h"
 
@@ -106,7 +105,7 @@ init_nntp_connection_info(struct connection *conn)
 
 	nntp = mem_calloc(1, sizeof(*nntp));
 	if (!nntp) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return NULL;
 	}
 
@@ -144,7 +143,7 @@ init_nntp_connection_info(struct connection *conn)
 			break;
 
 		/* FIXME: Special S_NNTP_BAD_RANGE */
-		abort_conn_with_state(conn, S_BAD_URL);
+		abort_connection(conn, S_BAD_URL);
 		return NULL;
 
 	case NNTP_TARGET_ARTICLE_NUMBER:
@@ -165,13 +164,13 @@ init_nntp_connection_info(struct connection *conn)
 			    || !redirect_cache(conn->cached, "/", 0, 0))
 				state = S_OUT_OF_MEM;
 
-			abort_conn_with_state(conn, state);
+			abort_connection(conn, state);
 			return NULL;
 		}
 
 		/* Reject nntp://<server>/<group>/<group> */
 		if (nntp->group.source)	{
-			abort_conn_with_state(conn, S_BAD_URL);
+			abort_connection(conn, S_BAD_URL);
 			return NULL;
 		}
 
@@ -197,7 +196,7 @@ nntp_quit(struct connection *conn)
 
 	info = mem_calloc(1, sizeof(*info));
 	if (!info) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
@@ -214,7 +213,7 @@ nntp_end_request(struct connection *conn, enum connection_state state)
 	struct nntp_connection_info *nntp = conn->info;
 
 	if (nntp->target == NNTP_TARGET_QUIT) {
-		abort_conn_with_state(conn, state);
+		abort_connection(conn, state);
 		return;
 	}
 
@@ -222,8 +221,7 @@ nntp_end_request(struct connection *conn, enum connection_state state)
 
 	if (conn->state == S_OK) {
 		if (conn->cached) {
-			truncate_entry(conn->cached, conn->from, 1);
-			conn->cached->incomplete = 0;
+			normalize_cache_entry(conn->cached, conn->from);
 		}
 	} else if (conn->state == S_OUT_OF_MEM) {
 		/* FIXME: Clear the socket buffers before ending so the one
@@ -237,11 +235,11 @@ nntp_end_request(struct connection *conn, enum connection_state state)
 /* Reponse receiving: */
 
 static void
-read_nntp_data(struct connection *conn, struct read_buffer *rb)
+read_nntp_data(struct socket *socket, struct read_buffer *rb)
 {
-	set_connection_timeout(conn);
+	struct connection *conn = socket->conn;
 
-	if (rb->close == READ_BUFFER_END) {
+	if (socket->state == SOCKET_CLOSED) {
 		nntp_end_request(conn, S_OK);
 		return;
 	}
@@ -257,8 +255,7 @@ read_nntp_data(struct connection *conn, struct read_buffer *rb)
 
 	case S_TRANS:
 	default:
-		read_from_socket(conn, &conn->socket, rb, read_nntp_data);
-		set_connection_state(conn, S_TRANS);
+		read_from_socket(conn->socket, rb, S_TRANS, read_nntp_data);
 	}
 }
 
@@ -296,13 +293,12 @@ get_nntp_connection_state(enum nntp_code code)
 }
 
 static void
-nntp_got_response(struct connection *conn, struct read_buffer *rb)
+nntp_got_response(struct socket *socket, struct read_buffer *rb)
 {
+	struct connection *conn = socket->conn;
 	struct nntp_connection_info *nntp = conn->info;
 
-	set_connection_timeout(conn);
-
-	if (rb->close == READ_BUFFER_END) {
+	if (socket->state == SOCKET_CLOSED) {
 		nntp_end_request(conn, S_OK);
 		return;
 	}
@@ -311,8 +307,7 @@ nntp_got_response(struct connection *conn, struct read_buffer *rb)
 
 	switch (nntp->code) {
 	case NNTP_CODE_NONE:
-		read_from_socket(conn, &conn->socket, rb, nntp_got_response);
-		set_connection_state(conn, S_TRANS);
+		read_from_socket(conn->socket, rb, S_TRANS, nntp_got_response);
 		break;
 
 	case NNTP_CODE_INVALID:
@@ -337,7 +332,7 @@ nntp_got_response(struct connection *conn, struct read_buffer *rb)
 	case NNTP_CODE_224_FOLLOW_XOVER:
 	case NNTP_CODE_230_FOLLOW_NEWNEWS:
 	case NNTP_CODE_231_FOLLOW_NEWGROUPS:
-		read_nntp_data(conn, rb);
+		read_nntp_data(socket, rb);
 		break;
 
 	case NNTP_CODE_500_COMMAND_UNKNOWN:
@@ -356,16 +351,15 @@ nntp_got_response(struct connection *conn, struct read_buffer *rb)
 }
 
 static void
-nntp_get_response(struct connection *conn)
+nntp_get_response(struct socket *socket)
 {
-	struct read_buffer *rb = alloc_read_buffer(conn);
+	struct connection *conn = socket->conn;
+	struct read_buffer *rb = alloc_read_buffer(conn->socket);
 
 	if (!rb) return;
 
-	set_connection_timeout(conn);
-
-	rb->close = READ_BUFFER_END_ONCLOSE;
-	read_from_socket(conn, &conn->socket, rb, nntp_got_response);
+	socket->state = SOCKET_END_ONCLOSE;
+	read_from_socket(conn->socket, rb, conn->state, nntp_got_response);
 }
 
 
@@ -488,7 +482,7 @@ add_nntp_command_to_string(struct string *req, struct nntp_connection_info *nntp
 
 		assert(nntp->current_article <= nntp->end_article);
 
-		add_format_to_string(req, "HEAD %d", nntp->current_article++);
+		add_format_to_string(req, "HEAD %ld", nntp->current_article++);
 		break;
 
 	case NNTP_COMMAND_QUIT:
@@ -528,9 +522,9 @@ nntp_send_command(struct connection *conn)
 	/* FIXME: Check non empty and < NNTP_MAX_COMMAND_LENGTH */
 	add_nntp_command_to_string(&req, nntp);
 
-	write_to_socket(conn, &conn->socket, req.source, req.length, nntp_get_response);
+	request_from_socket(conn->socket, req.source, req.length, S_SENT,
+			    SOCKET_END_ONCLOSE, nntp_got_response);
 	done_string(&req);
-	set_connection_state(conn, S_SENT);
 }
 
 
@@ -542,10 +536,9 @@ nntp_protocol_handler(struct connection *conn)
 	if (!init_nntp_connection_info(conn))
 		return;
 
-	set_connection_timeout(conn);
-
 	if (!has_keepalive_connection(conn)) {
-		make_connection(conn, &conn->socket, nntp_get_response);
+		make_connection(conn->socket, conn->uri, nntp_get_response,
+				conn->cache_mode >= CACHE_MODE_FORCE_RELOAD);
 	} else {
 		nntp_send_command(conn);
 	}
@@ -575,13 +568,13 @@ news_protocol_handler(struct connection *conn)
 
 	if (!*server) server = getenv("NNTPSERVER");
 	if (!server || !*server) {
-		abort_conn_with_state(conn, S_NNTP_NEWS_SERVER);
+		abort_connection(conn, S_NNTP_NEWS_SERVER);
 		return;
 	}
 
 	conn->cached = get_cache_entry(conn->uri);
 	if (!conn->cached || !init_string(&location)) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
@@ -594,5 +587,5 @@ news_protocol_handler(struct connection *conn)
 
 	done_string(&location);
 
-	abort_conn_with_state(conn, S_OK);
+	abort_connection(conn, S_OK);
 }
