@@ -1,10 +1,12 @@
 /* Internal SMB protocol implementation */
-/* $Id: smb.c,v 1.66.4.1 2005/04/05 21:08:42 jonas Exp $ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE /* Needed for asprintf() */
 #endif
 
+#error SMB protocol support is vulnerable to CVE-2006-5925.  Do not use.
+#error If you want to use SMB, please wait for ELinks 0.12.0 or see bug 844.
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -13,13 +15,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h> /* FreeBSD needs this before resource.h */
-#endif
-#include <sys/types.h> /* FreeBSD needs this before resource.h */
-#ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#endif
+#include <sys/types.h>
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h> /* OS/2 needs this after sys/types.h */
 #endif
@@ -32,14 +28,15 @@
 #include "cache/cache.h"
 #include "config/options.h"
 #include "intl/gettext/libintl.h"
-#include "lowlevel/connect.h"
-#include "lowlevel/select.h"
-#include "modules/module.h"
+#include "main/module.h"
+#include "main/select.h"
+#include "network/connection.h"
+#include "network/socket.h"
 #include "osdep/osdep.h"
+#include "protocol/common.h"
 #include "protocol/protocol.h"
 #include "protocol/smb/smb.h"
 #include "protocol/uri.h"
-#include "sched/connection.h"
 #include "util/memory.h"
 #include "util/snprintf.h"
 #include "util/string.h"
@@ -62,7 +59,7 @@ struct smb_connection_info {
 	 * should call end_smb_connection() when it goes off as well. */
 	int closing;
 
-	int textlen;
+	size_t textlen;
 	unsigned char text[1];
 };
 
@@ -101,28 +98,28 @@ smb_get_cache(struct connection *conn)
 	conn->cached = get_cache_entry(conn->uri);
 	if (conn->cached) return 0;
 
-	abort_conn_with_state(conn, S_OUT_OF_MEM);
+	abort_connection(conn, S_OUT_OF_MEM);
 	return -1;
 }
 
 
 #define READ_SIZE	4096
 
-static int
+static ssize_t
 smb_read_data(struct connection *conn, int sock, unsigned char *dst)
 {
-	int r;
+	ssize_t r;
 	struct smb_connection_info *si = conn->info;
 
 	r = read(sock, dst, READ_SIZE);
 	if (r == -1) {
-		retry_conn_with_state(conn, -errno);
+		retry_connection(conn, -errno);
 		return -1;
 	}
 	if (r == 0) {
 		if (!si->closing) {
 			si->closing = 1;
-			set_handlers(sock, NULL, NULL, NULL, NULL);
+			clear_handlers(sock);
 			return 0;
 		}
 		end_smb_connection(conn);
@@ -135,7 +132,7 @@ smb_read_data(struct connection *conn, int sock, unsigned char *dst)
 static void
 smb_read_text(struct connection *conn, int sock)
 {
-	int r;
+	ssize_t r;
 	struct smb_connection_info *si = conn->info;
 
 	/* We add 2 here to handle LF and NUL chars that are added in
@@ -143,7 +140,7 @@ smb_read_text(struct connection *conn, int sock)
 	si = mem_realloc(si, sizeof(*si) + si->textlen
 			     + READ_SIZE + 2);
 	if (!si) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 	conn->info = si;
@@ -160,14 +157,14 @@ smb_got_data(struct connection *conn)
 {
 	struct smb_connection_info *si = conn->info;
 	unsigned char buffer[READ_SIZE];
-	int r;
+	ssize_t r;
 
 	if (si->list_type != SMB_LIST_NONE) {
-		smb_read_text(conn, conn->data_socket.fd);
+		smb_read_text(conn, conn->data_socket->fd);
 		return;
 	}
 
-	r = smb_read_data(conn, conn->data_socket.fd, buffer);
+	r = smb_read_data(conn, conn->data_socket->fd, buffer);
 	if (r <= 0) return;
 
 	set_connection_state(conn, S_TRANS);
@@ -185,7 +182,7 @@ smb_got_data(struct connection *conn)
 static void
 smb_got_text(struct connection *conn)
 {
-	smb_read_text(conn, conn->socket.fd);
+	smb_read_text(conn, conn->socket->fd);
 }
 
  /* Search for @str1 followed by @str2 in @line.
@@ -214,7 +211,7 @@ parse_smbclient_output(struct uri *uri, struct smb_connection_info *si,
 		SMB_TYPE_SERVER,
 		SMB_TYPE_WORKGROUP
 	} type = SMB_TYPE_NONE;
-	int pos = 0;
+	size_t pos = 0;
 	int stop;
 
 	assert(uri && si && page);
@@ -228,8 +225,8 @@ parse_smbclient_output(struct uri *uri, struct smb_connection_info *si,
 	stop = !si->textlen;	/* Nothing to parse. */
 	while (!stop && (line_end = strchr(line_start, ASCII_LF))) {
 		unsigned char *line = line_start;
-		int line_len;
-		int start_offset = 0;
+		size_t line_len;
+		size_t start_offset = 0;
 
 		/* Handle \r\n case. Normally, do not occur on *nix. */
 		if (line_end > line_start && line_end[-1] == ASCII_CR)
@@ -482,8 +479,7 @@ end_smb_connection(struct connection *conn)
 	if (smb_get_cache(conn)) return;
 
 	if (conn->from) {
-		truncate_entry(conn->cached, conn->from, 1);
-		conn->cached->incomplete = 0;
+		normalize_cache_entry(conn->cached, conn->from);
 		goto bye;
 	}
 
@@ -514,36 +510,19 @@ end_smb_connection(struct connection *conn)
 
 		add_fragment(conn->cached, 0, page.source, page.length);
 		conn->from += page.length;
-		truncate_entry(conn->cached, page.length, 1);
-		conn->cached->incomplete = 0;
+		normalize_cache_entry(conn->cached, page.length);
 		done_string(&page);
 
 		mem_free_set(&conn->cached->content_type, stracpy("text/html"));
 	}
 
 bye:
-	close_socket(conn, &conn->socket);
-	close_socket(conn, &conn->data_socket);
-	abort_conn_with_state(conn, state);
+	close_socket(conn->socket);
+	close_socket(conn->data_socket);
+	abort_connection(conn, state);
 }
 
 
-/* Close all non-terminal file descriptors. */
-static void
-close_all_non_term_fd(void)
-{
-	int n;
-	int max = 1024;
-#ifdef RLIMIT_NOFILE
-	struct rlimit lim;
-
-	if (!getrlimit(RLIMIT_NOFILE, &lim))
-		max = lim.rlim_max;
-#endif
-	for (n = 3; n < max; n++)
-		close(n);
-}
-
 void
 smb_protocol_handler(struct connection *conn)
 {
@@ -558,7 +537,7 @@ smb_protocol_handler(struct connection *conn)
 
 	si = mem_calloc(1, sizeof(*si) + 2);
 	if (!si) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 	conn->info = si;
@@ -574,7 +553,7 @@ smb_protocol_handler(struct connection *conn)
 		if (smb_get_cache(conn)) return;
 
 		redirect_cache(conn->cached, "/", 1, 0);
-		abort_conn_with_state(conn, S_OK);
+		abort_connection(conn, S_OK);
 		return;
 
 	} else {
@@ -583,7 +562,7 @@ smb_protocol_handler(struct connection *conn)
 	}
 
 	if (!share) {
-		abort_conn_with_state(conn, S_OUT_OF_MEM);
+		abort_connection(conn, S_OUT_OF_MEM);
 		return;
 	}
 
@@ -601,7 +580,7 @@ smb_protocol_handler(struct connection *conn)
 		if (out_pipe[0] >= 0) close(out_pipe[0]);
 		if (out_pipe[1] >= 0) close(out_pipe[1]);
 		mem_free(share);
-		abort_conn_with_state(conn, -s_errno);
+		abort_connection(conn, -s_errno);
 		return;
 	}
 
@@ -616,7 +595,7 @@ smb_protocol_handler(struct connection *conn)
 		close(err_pipe[0]);
 		close(err_pipe[1]);
 		mem_free(share);
-		retry_conn_with_state(conn, -s_errno);
+		retry_connection(conn, -s_errno);
 		return;
 	}
 
@@ -733,13 +712,13 @@ smb_protocol_handler(struct connection *conn)
 
 	mem_free(share);
 
-	conn->data_socket.fd = out_pipe[0];
-	conn->socket.fd = err_pipe[0];
+	conn->data_socket->fd = out_pipe[0];
+	conn->socket->fd = err_pipe[0];
 
 	close(out_pipe[1]);
 	close(err_pipe[1]);
 
-	set_handlers(out_pipe[0], (void (*)(void *)) smb_got_data, NULL, NULL, conn);
-	set_handlers(err_pipe[0], (void (*)(void *)) smb_got_text, NULL, NULL, conn);
+	set_handlers(out_pipe[0], (select_handler_T) smb_got_data, NULL, NULL, conn);
+	set_handlers(err_pipe[0], (select_handler_T) smb_got_text, NULL, NULL, conn);
 	set_connection_state(conn, S_CONN);
 }
